@@ -1,6 +1,6 @@
 /*
     SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
-    SPDX-FileCopyrightText: 2021 Harald Sitter <sitter@kde.org>
+    SPDX-FileCopyrightText: 2021-2022 Harald Sitter <sitter@kde.org>
 */
 
 #include "CommandOutputContext.h"
@@ -12,12 +12,14 @@
 #include <QStandardPaths>
 
 #include <KLocalizedString>
+#include <KOSRelease>
 
-CommandOutputContext::CommandOutputContext(const QString &executable, const QStringList &arguments, QObject *parent)
+CommandOutputContext::CommandOutputContext(const QStringList &findExecutables, const QString &executable, const QStringList &arguments, QObject *parent)
     : QObject(parent)
     , m_executableName(executable)
     , m_executablePath(QStandardPaths::findExecutable(m_executableName))
     , m_arguments(arguments)
+    , m_bugReportUrl(KOSRelease().bugReportUrl())
 {
     // Various utilities are installed in sbin, but work without elevated privileges
     if (m_executablePath.isEmpty()) {
@@ -25,9 +27,19 @@ CommandOutputContext::CommandOutputContext(const QString &executable, const QStr
             QStandardPaths::findExecutable(m_executableName, {QStringLiteral("/usr/local/sbin"), QStringLiteral("/usr/sbin"), QStringLiteral("/sbin")});
     }
 
+    m_foundExecutablePaths[executable] = m_executablePath;
+    for (const QString &findExecutable : findExecutables) {
+        m_foundExecutablePaths[findExecutable] = QStandardPaths::findExecutable(findExecutable);
+    }
+
     metaObject()->invokeMethod(this, &CommandOutputContext::load);
 }
 
+CommandOutputContext::CommandOutputContext(const QString &executable, const QStringList &arguments, QObject *parent)
+    : CommandOutputContext({/* executable is by default always searched for */}, executable, arguments, parent)
+{
+}
+
 QString CommandOutputContext::executableName() const
 {
     return m_executableName;
@@ -64,10 +76,12 @@ void CommandOutputContext::reset()
 {
     m_ready = false;
     m_error.clear();
+    m_explanation.clear();
     m_text.clear();
     m_filter.clear();
     Q_EMIT readyChanged();
     Q_EMIT errorChanged();
+    Q_EMIT explanationChanged();
     Q_EMIT textChanged();
     Q_EMIT filterChanged();
 
@@ -79,9 +93,14 @@ void CommandOutputContext::load()
 {
     reset();
 
-    if (m_executablePath.isEmpty()) {
-        setError(xi18nc("@info", "The executable <command>%1</command> could not be found in $PATH.", m_executableName));
-        return;
+    for (auto it = m_foundExecutablePaths.cbegin(); it != m_foundExecutablePaths.cend(); ++it) {
+        if (it.value().isEmpty()) {
+            setError(xi18nc("@info", "The <command>%1</command> tool is required to display this page, but could not be found", it.key()),
+                     xi18nc("@info",
+                            "You can search for it and install it your using your package manager.<nl/>"
+                            "Then please report this packaging issue to your distribution."));
+            return;
+        }
     }
 
     auto proc = new QProcess(this);
@@ -91,7 +110,8 @@ void CommandOutputContext::load()
 
         switch (exitStatus) {
         case QProcess::CrashExit:
-            return setError(xi18nc("@info", "The subprocess <command>%1</command> crashed unexpectedly. No data could be obtained.", m_executableName));
+            return setError(xi18nc("@info", "The <command>%1</command> tool crashed while generating page content", m_executableName),
+                            xi18nc("@Info", "Try again later. If keeps happening, please report the crash to your distribution."));
         case QProcess::NormalExit:
             break;
         }
@@ -110,10 +130,17 @@ void CommandOutputContext::load()
     proc->start(m_executablePath, m_arguments);
 }
 
-void CommandOutputContext::setError(const QString &message)
+void CommandOutputContext::setError(const QString &message, const QString &explanation = QString())
 {
     m_error = message;
+
+    if (!explanation.isEmpty()) {
+        m_explanation = explanation;
+    }
+
     Q_EMIT errorChanged();
+    Q_EMIT explanationChanged();
+
     setReady();
 }
 
